
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "xstd/TblDistr.h"
#include "xstd/StrIdentifier.h"
#include "pgl/PglRec.h"
#include "pgl/PglDistrSym.h"
#include "pgl/PglStringSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/SocketSym.h"
#include "pgl/UniqIdSym.h"
#include "pgl/PopModelSym.h"
#include "pgl/AgentSym.h"


AgentSym::AgentSym(const String &aType): HostsBasedSym(aType) {
	theRec->bAdd("Socket", "socket", new SocketSym);

	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("uniq_id", "world", 0);
	theRec->bAdd("time_distr", "xact_think", 0);
	theRec->bAdd("int_distr", "pconn_use_lmt", 0);
	theRec->bAdd("time", "idle_pconn_tout", 0);
	theRec->bAdd("PopModel", "pop_model", new PopModelSym);
	theRec->bAdd("float", "abort_prob", 0);
}

AgentSym::AgentSym(const String &aType, PglRec *aRec): HostsBasedSym(aType, aRec) {
}

bool AgentSym::isA(const String &type) const {
	return HostsBasedSym::isA(type) || type == "Agent";
}

String AgentSym::kind() const {
	return getString("kind");
}

UniqId AgentSym::world() const {
	SynSymTblItem *wi = 0;
	Assert(theRec->find("world", wi));
	return wi->sym() ?
		((const UniqIdSym&)wi->sym()->cast("uniq_id")).val() : UniqId();
}

SocketSym *AgentSym::socket() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("socket", si));
	Assert(si->sym());
	return &(SocketSym&)si->sym()->cast("Socket");
}

RndDistr *AgentSym::xactThink() const {
	return getDistr("xact_think");
}

RndDistr *AgentSym::pconnUseLmt() const {
	return getDistr("pconn_use_lmt");
}

Time AgentSym::idlePconnTimeout() const {
	return getTime("idle_pconn_tout");
}

PopModelSym *AgentSym::popModel() const {
	const SynSym *pms = getRecSym("pop_model");
	return pms ?
		&((PopModelSym&)pms->cast("PopModel")) : 0;
}

RndDistr *AgentSym::namesToDistr(const String &fieldName, const StrIdentifier &names) const {
	Assert(names.count());

	SynSymTblItem *fti = 0;
	Assert(theRec->find(fieldName, fti));
	if (!fti->sym())
		return 0; // undefined

	// build selector using label index
	ArraySym &nts = (ArraySym&)fti->sym()->cast("Array");
	Array<double> probs_in(names.count());
	Array<double> probs_out(names.count());
	nts.copyProbs(probs_in);

	for (int p = 0; p < probs_in.count(); ++p) {
		const String &name = ((const StringSym&)nts[p]->cast("string")).val();

		const int id = names.lookup(name);
		if (id <= 0) {
			cerr << loc() << "unknown quoted name `" << name
				<< "'; known names within " << fieldName << " context are:";
			for (StrIdentifier::Iter i = names.iterator(); i; ++i)
				cerr << " \"" << i.str() << '"';
			cerr << endl;
			exit(-2);
		}

		if (probs_in[p] > 0)
			probs_out.put(probs_in[p], id);
	}

	if (!probs_out.count())
		return 0;

	return TblDistr::FromDistrTable(kind() + "-" + fieldName, probs_out);
}

bool AgentSym::msgTypes(Array<StringSym*> &types, Array<double> &tprobs) const {
	if (ArraySym *a = getArraySym(msgTypesField())) {
		a->copyProbs(tprobs);
		ArraySymExportM(StringSym, *a, "String", types);
		return true;
	}
	return false;
}

RndDistr *AgentSym::msgTypes(const StrIdentifier &typeNames) const {
	return namesToDistr(msgTypesField(), typeNames);
}

bool AgentSym::abortProb(double &pct) const {
        return getDouble("abort_prob", pct);
}
