
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglDistrSym.h"
#include "pgl/ObjLifeCycleSym.h"


ObjLifeCycleSym::ObjLifeCycleSym(): RecSym("ObjLifeCycle", new PglRec) {

	// add fields

	theRec->bAdd("time_distr", "birthday", 0);
	theRec->bAdd("time_distr", "length", 0);
	theRec->bAdd("float", "variance", 0);
	theRec->bAdd("float", "with_lmt", 0);
	theRec->bAdd("time_distr[]", "expires", 0);
}

ObjLifeCycleSym::ObjLifeCycleSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
	// find fields
}

bool ObjLifeCycleSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "ObjLifeCycle";
}

SynSym *ObjLifeCycleSym::dupe(const String &type) const {
	if (isA(type))
		return new ObjLifeCycleSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

RndDistr *ObjLifeCycleSym::bday() const {
	return getDistr("birthday");
}

RndDistr *ObjLifeCycleSym::length() const {
	return getDistr("length");
}

bool ObjLifeCycleSym::variance(double &ratio) const {
	return getDouble("variance", ratio);
}

bool ObjLifeCycleSym::withLmt(double &ratio) const {
	return getDouble("with_lmt", ratio);
}

bool ObjLifeCycleSym::expires(Array<RndDistr*> &times, RndDistr *&selector, Array<QualifSym*> &qs) const {
	SynSymTblItem *ti = 0;
	Assert(theRec->find("expires", ti));
	if (!ti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)ti->sym()->cast("Array");
	selector = a.makeSelector("expires");
	for (int i = 0; i < a.count(); ++i) {
		DistrSym &ds = (DistrSym&)a[i]->cast("Distr");
		times.append(ds.val());
		qs.append(ds.qualifier());
	}

	return true;
}
