
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "pgl/PglListSym.h"


ListSym::ListSym(const String &aType): ExpressionSym(aType) {
}

ListSym::~ListSym() {
	while (count()) delete pop();
}

bool ListSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "List";
}

SynSym *ListSym::dupe(const String &type) const {
	if (!isA(type))
		return 0;

	ListSym *clone = new ListSym("List");

	if (!clone->absorb(*this)) {
		delete clone;
		return 0;
	}

	return clone;
}

void ListSym::add(const SynSym &s) {
	if (!cadd(s))
		addFailure(s);
}

void ListSym::addFailure(const SynSym &) {
	Assert(0);
}

bool ListSym::cadd(const SynSym &s) {
	doAdd(s.clone());
	return true;
}

void ListSym::clear() {
	while (count()) delete pop();		
}

void ListSym::doAdd(SynSym *sclone) {
	Assert(sclone);
	append(sclone);
}

bool ListSym::absorb(const ListSym &arr) {
	const int scount = count();

	for (int i = 0; i < arr.count(); ++i) {
		Assert(arr[i]);
		if (!cadd(*arr[i])) {
			// restore previous state
			while (count() > scount) delete pop();
			return false;
		}
	}

	return true;
}

ostream &ListSym::print(ostream &os, const String &pfx) const {
	os << '[';
	for (int i = 0; i < count(); ++i) {
		if (i) os << ',';
		item(i)->print(os, pfx+"    ");
	}
	return os << ']';
}
