
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "pgl/PglRec.h"
#include "pgl/PglStringSym.h"
#include "pgl/GoalSym.h"
#include "pgl/SessionSym.h"


SessionSym::SessionSym(): RecSym("Session", new PglRec) {
	theRec->bAdd("Goal", "busy_period", new GoalSym);
	theRec->bAdd("time_distr", "idle_period_duration", 0);
	theRec->bAdd("rate", "heartbeat_notif_rate", 0);
}

SessionSym::SessionSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool SessionSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Session";
}

SynSym *SessionSym::dupe(const String &type) const {
	if (isA(type))
		return new SessionSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

GoalSym *SessionSym::busyPeriod() const {
	SynSymTblItem *gi = 0;
	Assert(theRec->find("busy_period", gi));
	Assert(gi->sym());
	return &(GoalSym&)gi->sym()->cast("Goal");
}

RndDistr *SessionSym::idlePeriodDuration() const {
	return getDistr("idle_period_duration");
}

bool SessionSym::heartbeatGap(Time &gap) const {
	double rate;
	if (getRate("heartbeat_notif_rate", rate)) {
		gap = rate > 0 ? Time::Secd(1/rate) : Time();
		return true;
	}
	return false;
}
