
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/math.h"

#include "runtime/StatPhase.h"
#include "runtime/TransFactor.h"

TransFactor::TransFactor(const StatPhase *aPhase): thePhase(aPhase),
	theBeg(1), theEnd(1), theK(0) {
	Assert(thePhase);
}

// note: simply left some values unchanged on failure
// the caller will check and complain
void TransFactor::configure(double beg, double end, const TransFactor *prev) {
	if (beg >= 0)
		theBeg = beg;
	else
	if (prev) // use the previous end factor if needed
		theBeg = prev->theEnd;

	theEnd = end >= 0 ? end : theBeg;	

	const double fdiff = theEnd - theBeg;
	
	if (fdiff && !goal())
		return;

	// cache often needed value
	// note: duration takes priority over xact count if both are given
	theK = 
		goal().duration() > 0 ?
			fdiff/goal().duration().secd() :
		(goal().xactCnt() > 0 ?
			fdiff/goal().xactCnt() : 0);
}

const GoalRec &TransFactor::goal() const {
	return thePhase->goal();
}

double TransFactor::current() const {
	const double t = goal().duration() > 0 ?
		thePhase->duration().secd() : (double)thePhase->xactCnt();

	const double f = theBeg + theK*t;
	const double d = f*f + 4*theK;
	const double newF = d >= 0 ? (sqrt(d) + f)/2 : (f/2);

	// make sure load factor stays between beg and end
	return theBeg <= theEnd ?
		MiniMax(theBeg, newF, theEnd):
		MiniMax(theEnd, newF, theBeg);
}

void TransFactor::change(double delta) {
	if (flat()) {
		theBeg += delta*theBeg;
		theEnd = theBeg;
	} else {
		theK += delta*theK;
	}
}
